.used-prog-language {
  flex: 5;
  margin: 0.5rem 1rem;
  text-align: end;
}

.used-prog-language span {
  background-color: #00262b;
  color: white;
  display: inline-block;
  padding: .25em .4em;
  font-size: 75%;
  font-weight: 700;
  line-height: 1;
  text-align: center;
  white-space: nowrap;
  vertical-align: baseline;
  border-radius: .25rem;
}

.eval-ai-container {
  display: flex;
  max-height: 600px;
  box-shadow: 0px 0px 10px 0px #0000001A;
  border: 1px solid #CED4DA;
  border-radius: 4px;
}

.eval-ai-container .eval-ai-code-editor {
  flex: 5;
}

.eval-ai-container #monaco {
  min-height: 500px;
  width: 100%;
  border-bottom: 1.5px solid #CED4DA;
}

.eval-ai-container .result {
  flex: 3;
  background-color: #f5f5f5;
  overflow-y: hidden;
  border-top-right-radius: 4px;
  border-bottom-right-radius: 4px;
}

.eval-ai-container .html-render {
  height: 100vh;
}

.eval-ai-container .eval-ai-buttons {
  display: flex;
  justify-content: flex-end;
}

.eval-ai-container .submit-loader {
  margin-left: 5px;
}

.eval-ai-container .eval-ai-buttons .eval-ai-button {
  margin: 10px;
  text-align: center;
  height: fit-content;
  padding: 8px 12px;
  border-radius: 4px;
  font-size: 0.875em;
  line-height: 1.25em;
}

.eval-ai-container .eval-ai-buttons .disabled-btn {
  opacity: 0.8;
  cursor: not-allowed !important;
}

.eval-ai-container .stderr {
  margin-top: 10px;
  color: red;
  white-space: pre-wrap;
}

.eval-ai-container .stdout {
  white-space: pre-wrap;
}

/* tabs */
.tab {
  display: flex;
  border-bottom: 1px solid #edeae9;
}

.tab .result-tab-btn {
  float: left;
  border: none;
  outline: none;
  cursor: pointer;
  padding: 8px 12px;
  transition: 0.3s;
  font-size: 0.875em;
  line-height: 1.25em;
}

.tab .result-tab-btn:hover {
  background-color: #ddd;
}

.tab .result-tab-btn.active {
  border-bottom: 2px solid #546d70;
}

.tabcontent {
  padding: 6px 12px;
  border-top: none;
  animation: fadeEffect 1s;
  overflow-y: scroll;
  height: calc(100% - 48px);
}

@keyframes fadeEffect {
  from {
    opacity: 0;
  }

  to {
    opacity: 1;
  }
}