<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8" />
  <title>Monaco Editor</title>
</head>
<body style="margin: 0;">
  <div class="monaco" style="width: 100vw; height: 100vh;"></div>
  <script type="module">
    import * as monaco from "https://cdn.jsdelivr.net/npm/monaco-editor@0.49.0/+esm";

    window.editor = monaco.editor.create(document.querySelector(".monaco"), {
      language: "{{ monaco_language }}",
      minimap: { enabled: false },
      lineNumbersMinChars: 2,
      folding: false,
    });

    window.parent.postMessage('__USAGE_ID_PLACEHOLDER__', '*');
  </script>
</body>
</html>
